/*
 * <aisl/stream.h>
 *
 * Copyright (c) 2017-2019 by Löwenware Ltd.
 *
 * Project homepage: https://lowenware.com/aisl/
 *
 */

#ifndef STREAM_H_4D8EB622_3CE0_4F1B_AC1F_B27CCB5C2EDC
#define STREAM_H_4D8EB622_3CE0_4F1B_AC1F_B27CCB5C2EDC

typedef struct aisl_stream * aisl_stream_t;


/* Stream helpers */

bool
aisl_is_secure( aisl_stream_t stream );


aisl_client_t
aisl_get_client( aisl_stream_t stream );


aisl_server_t
aisl_get_server( aisl_stream_t stream );


aisl_http_version_t
aisl_get_http_version( aisl_stream_t stream );


aisl_instance_t
aisl_stream_get_instance( aisl_stream_t s );


/* Context functions */

void *
aisl_get_context( aisl_stream_t stream );


void
aisl_set_context( aisl_stream_t stream, void * context );


/* Stream control functions */

aisl_status_t
aisl_flush( aisl_stream_t stream );


void
aisl_cancel( aisl_stream_t stream );


void
aisl_reject( aisl_stream_t stream );


/* Response functions */

aisl_status_t
aisl_response( aisl_stream_t          stream,
               aisl_http_response_t   status_code,
               const char           * content_type,
               uint32_t               content_length);


int
aisl_header( aisl_stream_t stream, const char *key, const char *value );


int
aisl_header_printf( aisl_stream_t   stream, 
                    const char    * key,
                    const char    * format,
                    ... );


int
aisl_header_vprintf( aisl_stream_t   stream,
                     const char    * key,
                     const char    * format,
                     va_list         args );


int
aisl_printf( aisl_stream_t stream, const char * format, ... );


int
aisl_vprintf( aisl_stream_t stream, const char * format, va_list args );


int
aisl_write( aisl_stream_t stream, const char * data, int d_len );


int
aisl_puts( const char * str_data, aisl_stream_t stream );

#endif /* !AISL_STREAM_H */
